@@ -553,28 +553,19 @@ namespace Exif {
 		
 		public void SetData (byte []data)
 		{
-			SetData (data, data.Length);
+			SetData (data, 1);
 		}
 
 		public void SetData (uint s)
 		{
-			this.SetData (FSpot.BitConverter.GetBytes (s, this.ByteOrder == ByteOrder.Intel));
+			this.SetData (FSpot.BitConverter.GetBytes (s, this.ByteOrder == ByteOrder.Intel), 4);
 		}
 
 		public void SetData (ushort s)
 		{
-			this.SetData (FSpot.BitConverter.GetBytes (s, this.ByteOrder == ByteOrder.Intel));
+			this.SetData (FSpot.BitConverter.GetBytes (s, this.ByteOrder == ByteOrder.Intel), 2);
 		}	    
 
-		public void SetData (ushort [] data)
-		{
-		} 
-		
-		public void SetData (short [] data)
-		{
-			
-		}
-
 		public void SetData (string value)
 		{
 			int len = System.Text.Encoding.UTF8.GetByteCount (value);
