@@ -8,12 +8,12 @@ using SemWeb;
 // gracefully using ellipsis.
 
 public class InfoBox : VBox {
-
+	Delay update_delay;
 	private Photo photo;
 	public Photo Photo {
 		set {
 			photo = value;
-			Update ();
+			update_delay.Start ();
 		}
 
 		get {
@@ -124,8 +124,9 @@ public class InfoBox : VBox {
 
 		MemoryStore store;
 		
+#if USE_EXIF_DATE
 		DateTime date;
-
+#endif
 		public ImageInfo (ImageFile img) 
 		{
 			// FIXME We use the memory store to hold the anonymous statements
@@ -156,8 +157,9 @@ public class InfoBox : VBox {
 				width = real_width.ToString ();
 				height = real_height.ToString ();
 			}
-
+#if USE_EXIF_DATE
 			date = img.Date.ToLocalTime ();
+#endif
 		}
 
 		public bool Add (SemWeb.Statement stmt)
@@ -225,7 +227,7 @@ public class InfoBox : VBox {
 					return Mono.Posix.Catalog.GetString ("(Unknown)");
 			}
 		}
-
+#if USE_EXIF_DATE
 		public string Date {
 			get {
 				if (date > DateTime.MinValue && date < DateTime.MaxValue)
@@ -234,20 +236,20 @@ public class InfoBox : VBox {
 					return Mono.Posix.Catalog.GetString ("(Unknown)");
 			}
 		}
+#endif
 	}
 		
 
-	public void Update ()
+	public bool Update ()
 	{
 		ImageFile img;
 		ImageInfo info;
 
 		if (photo == null) {
 			Clear ();
-			return;
+			return false;
 		}
 		
-		
 		string path = photo.DefaultVersionUri.LocalPath;
 		name_entry.Text = System.IO.Path.GetFileName (path);
 		try {
@@ -264,8 +266,13 @@ public class InfoBox : VBox {
 		name_entry.Sensitive = true;
 		exposure_info_label.Text = info.ExposureInfo;
 		size_label.Text = info.Dimensions;
+#if USE_EXIF_DATE
 		date_label.Text = info.Date;
-
+#else
+		date_label.Text = String.Format ("{0}\n{1}",
+						 photo.Time.ToShortDateString (),
+						 photo.Time.ToShortTimeString ());
+#endif
 		version_option_menu.Sensitive = true;
 		PhotoVersionMenu menu = new PhotoVersionMenu (photo);
 		menu.VersionIdChanged += new PhotoVersionMenu.VersionIdChangedHandler (HandleVersionIdChanged);
@@ -281,6 +288,8 @@ public class InfoBox : VBox {
 			}
 			i++;
 		}
+
+		return false;
 	}
 
 
@@ -289,7 +298,8 @@ public class InfoBox : VBox {
 	public InfoBox () : base (false, 0)
 	{
 		SetupWidgets ();
-		Update ();
+		update_delay = new Delay (Update);
+		update_delay.Start ();
 
 		BorderWidth = 6;
 	}
