@@ -264,7 +264,7 @@ public class JpegHeader : SemWeb.StatementSource {
 		return null;
 	}
 	
-	private FSpot.Tiff.Header GetExifHeader ()
+	public FSpot.Tiff.Header GetExifHeader ()
 	{
 		string name = ExifSignature.Name;
 		Marker marker = FindMarker (ExifSignature);
@@ -272,9 +272,28 @@ public class JpegHeader : SemWeb.StatementSource {
 		if (marker == null)
 			return null;
 		
-		System.IO.Stream exifstream = new System.IO.MemoryStream (marker.Data, name.Length, marker.Data.Length - name.Length, false);
-		FSpot.Tiff.Header exif = new FSpot.Tiff.Header (exifstream);
-		return exif;
+		using (System.IO.Stream exifstream = new System.IO.MemoryStream (marker.Data, name.Length, marker.Data.Length - name.Length, false)) {
+			FSpot.Tiff.Header exif = new FSpot.Tiff.Header (exifstream);
+			return exif;
+		}
+	}
+
+	public XmpFile GetXmp ()
+	{
+		string name = XmpSignature.Name;
+		Marker marker = FindMarker (XmpSignature);
+		if (marker != null) {
+			int len = name.Length;
+			//System.Console.WriteLine (System.Text.Encoding.ASCII.GetString (marker.Data, len, 
+			//								marker.Data.Length - len));
+			using (System.IO.Stream xmpstream = new System.IO.MemoryStream (marker.Data, len, 
+											marker.Data.Length - len, false)) {
+			
+				XmpFile xmp = new XmpFile (xmpstream);					
+				return xmp;
+			}
+		}
+		return null;
 	}
 
 	public void Select (SemWeb.StatementSink sink)
@@ -283,26 +302,18 @@ public class JpegHeader : SemWeb.StatementSource {
 		if (exif != null)
 			exif.Select (sink);
 		
-		string name = XmpSignature.Name;
-		Marker marker = FindMarker (XmpSignature);
-		if (marker != null) {
-			int len = name.Length;
-			//System.Console.WriteLine (System.Text.Encoding.ASCII.GetString (marker.Data, len, 
-			//								marker.Data.Length - len));
-			System.IO.Stream xmpstream = new System.IO.MemoryStream (marker.Data, len, 
-										 marker.Data.Length - len, false);
-			
-			XmpFile xmp = new XmpFile (xmpstream);					
+		XmpFile xmp = GetXmp ();
+		if (xmp != null)
 			xmp.Select (sink);
-		}
 		
-		name = PhotoshopSignature.Name;
-		marker = FindMarker (PhotoshopSignature);
+		string name = PhotoshopSignature.Name;
+		JpegHeader.Marker marker = FindMarker (PhotoshopSignature);
 		if (marker != null) {
 			int len = name.Length;
-			System.IO.Stream bimstream = new System.IO.MemoryStream (marker.Data, len, marker.Data.Length - len, false);
-			FSpot.Bim.BimFile bim = new FSpot.Bim.BimFile (bimstream);
-					bim.Select (sink);
+			using (System.IO.Stream bimstream = new System.IO.MemoryStream (marker.Data, len, marker.Data.Length - len, false)) {
+				FSpot.Bim.BimFile bim = new FSpot.Bim.BimFile (bimstream);
+				bim.Select (sink);
+			}
 		}
 	}
 
@@ -353,14 +364,14 @@ public class JpegHeader : SemWeb.StatementSource {
 	
 	public void SetXmp (XmpFile xmp)
 	{
-		MemoryStream stream = new MemoryStream ();
-
-		XmpSignature.WriteName (stream);
-		xmp.Save (stream);
-
-		Marker xmp_marker = new Marker (XmpSignature.Id, stream.ToArray ());
-
-		Replace (XmpSignature, xmp_marker);
+		using (MemoryStream stream = new MemoryStream ()) {
+			
+			XmpSignature.WriteName (stream);
+			xmp.Save (stream);
+			
+			Marker xmp_marker = new Marker (XmpSignature.Id, stream.ToArray ());
+			Replace (XmpSignature, xmp_marker);
+		}
 	}
 	
 	public System.Collections.ArrayList Markers {
@@ -372,8 +383,8 @@ public class JpegHeader : SemWeb.StatementSource {
 	public void Save (System.IO.Stream stream)
 	{
 		foreach (Marker marker in marker_list) {
-			System.Console.WriteLine ("saving marker {0} {1}", marker.Type, 
-						   (marker.Data != null) ? marker.Data.Length .ToString (): "(null)");
+			//System.Console.WriteLine ("saving marker {0} {1}", marker.Type, 
+			//			   (marker.Data != null) ? marker.Data.Length .ToString (): "(null)");
 			marker.Save (stream);
 			if (marker.Type == JpegMarker.Sos)
 				stream.Write (ImageData, 0, ImageData.Length);
@@ -392,9 +403,10 @@ public class JpegHeader : SemWeb.StatementSource {
 
 	public JpegHeader (string filename) 
 	{
-		System.Console.WriteLine ("opening {0}", filename);
-		System.IO.FileStream stream = new System.IO.FileStream (filename, System.IO.FileMode.Open);
-		Load (stream, false);
+		//System.Console.WriteLine ("opening {0}", filename);
+		using (System.IO.FileStream stream = new System.IO.FileStream (filename, System.IO.FileMode.Open)) {
+			Load (stream, false);
+		}
 	}
 
 	private void Load (System.IO.Stream stream, bool metadata_only) 
