@@ -10,6 +10,7 @@ using System.Text;
 using System.Collections;
 using System.Runtime.InteropServices;
 using System.Text.RegularExpressions;
+using System.Web;
 using System.Web.Mail;
 
 using FSpot;
@@ -240,7 +241,8 @@ public class MainWindow {
 				  DragAction.Copy | DragAction.Move ); 
 
 		tag_selection_widget.ButtonPressEvent += HandleTagSelectionButtonPressEvent;
-
+		tag_selection_widget.PopupMenu += HandleTagSelectionPopupMenu;
+		
 		info_box = new InfoBox ();
 		info_box.VersionIdChanged += HandleInfoBoxVersionIdChange;
 		left_vbox.PackStart (info_box, false, true, 0);
@@ -291,6 +293,7 @@ public class MainWindow {
 		icon_view.DragDataGet += HandleIconViewDragDataGet;
 
 		TagMenu menu = new TagMenu (attach_tag, db.Tags);
+		menu.NewTagHandler = HandleCreateTagAndAttach;
 		menu.TagSelected += HandleAttachTagMenuSelected;
 
 		near_image.SetFromStock ("f-spot-stock_near", IconSize.SmallToolbar);
@@ -706,21 +709,27 @@ public class MainWindow {
 		}
 	}
 
-	[GLib.ConnectBefore]
 	void HandleTagSelectionButtonPressEvent (object sender, ButtonPressEventArgs args)
 	{
 		if (args.Event.Button == 3)
 		{
 			TagPopup popup = new TagPopup ();
 			popup.Activate (args.Event, tag_selection_widget.TagAtPosition ((int)args.Event.X, (int)args.Event.Y),
-			tag_selection_widget.TagHighlight ());
+			tag_selection_widget.TagHighlight);
 			args.RetVal = true;
 		}
 	}
 
+	void HandleTagSelectionPopupMenu (object sender, PopupMenuArgs args)
+	{
+		TagPopup popup = new TagPopup ();
+		popup.Activate (null, null, tag_selection_widget.TagHighlight);
+		args.RetVal = true;
+	}
+
 	void HandleTagSelectionDragBegin (object sender, DragBeginArgs args)
 	{
-		Tag [] tags = tag_selection_widget.TagHighlight ();
+		Tag [] tags = tag_selection_widget.TagHighlight;
 		int len = tags.Length;
 		int size = 32;
 		int csize = size/2 + len * size / 2 + 2;
@@ -814,7 +823,7 @@ public class MainWindow {
 			InvalidateViews ();
 			break;
 		case (uint)TargetType.TagList:
-			Tag child = tag_selection_widget.TagHighlight ()[0];
+			Tag child = tag_selection_widget.TagHighlight [0];
 			Tag parent = tags[0];
 
 			// FIXME with this reparenting via dnd, you cannot move a tag to root.
@@ -1064,9 +1073,9 @@ public class MainWindow {
 			//Console.WriteLine ("Drop cell = {0} ({1},{2})", item, args.X, args.Y);
 			if (item >= 0) {
 				if (icon_view.Selection.Contains (item))
-					AttachTags (tag_selection_widget.TagHighlight (), SelectedIds());
+					AttachTags (tag_selection_widget.TagHighlight, SelectedIds());
 				else 
-					AttachTags (tag_selection_widget.TagHighlight (), new int [] {item});
+					AttachTags (tag_selection_widget.TagHighlight, new int [] {item});
 			}
 			break;
 		case (uint)TargetType.UriList:
@@ -1391,9 +1400,8 @@ public class MainWindow {
 		StringBuilder url = new StringBuilder ("mailto:?subject=my%20photos");
 
 		foreach (Photo p in SelectedPhotos ()) {
-			url.Append ("&attach=" + p.DefaultVersionPath);
+			url.Append ("&attach=" + HttpUtility.UrlEncode(p.DefaultVersionPath));
 		}
-
 		GnomeUtil.UrlShow (main_window, url.ToString ());
 	}
 
@@ -1443,7 +1451,7 @@ public class MainWindow {
 
                 new About ("F-Spot", 
 			   FSpot.Defines.VERSION, 
-			   Mono.Posix.Catalog.GetString ("Copyright 2003-2006 Novell Inc."),
+			   Mono.Posix.Catalog.GetString ("Copyright \x00a9 2003-2006 Novell Inc."),
                            null, authors, new string [0], translators, null).Show();
 	}
 
@@ -1536,7 +1544,7 @@ public class MainWindow {
 		
 		Preferences.Set (Preferences.SIDEBAR_POSITION,		main_hpaned.Position);
 		Preferences.Set (Preferences.ZOOM,			icon_view.Zoom);
-	
+		
 		tag_selection_widget.SaveExpandDefaults ();
 
 		this.Window.Destroy ();
@@ -1583,23 +1591,34 @@ public class MainWindow {
 			query.MarkChanged (ActiveIndex ());
 		}
 	}
-
-	public void HandleCreateNewTagCommand (object sender, EventArgs args)
+	
+	public void HandleCreateTagAndAttach (object sender, EventArgs args)
 	{
-		TagCommands.Create command = new TagCommands.Create (db.Tags, main_window);
-		command.Execute (TagCommands.TagType.Tag, tag_selection_widget.TagHighlight ());
-		
+		Tag new_tag = CreateTag ();
+
+		if (new_tag != null)
+			HandleAttachTagMenuSelected (new_tag);
 	}
 
 	public void HandleCreateNewCategoryCommand (object sender, EventArgs args)
+	{
+		Tag new_tag = CreateTag ();
+		
+		if (new_tag != null) {
+			tag_selection_widget.ScrollTo (new_tag);
+			tag_selection_widget.TagHighlight = new Tag [] {new_tag};
+		}
+	}
+
+	public Tag CreateTag ()
 	{
 		TagCommands.Create command = new TagCommands.Create (db.Tags, main_window);
-		command.Execute (TagCommands.TagType.Category, tag_selection_widget.TagHighlight ());
+		return command.Execute (TagCommands.TagType.Category, tag_selection_widget.TagHighlight);
 	}
 
 	public void HandleAttachTagCommand (object obj, EventArgs args)
 	{
-		AttachTags (tag_selection_widget.TagHighlight (), SelectedIds ());
+		AttachTags (tag_selection_widget.TagHighlight, SelectedIds ());
 	}
 
 	void AttachTags (Tag [] tags, int [] ids) 
@@ -1613,7 +1632,7 @@ public class MainWindow {
 
 	public void HandleRemoveTagCommand (object obj, EventArgs args)
 	{
-		Tag [] tags = this.tag_selection_widget.TagHighlight ();
+		Tag [] tags = this.tag_selection_widget.TagHighlight;
 
 		db.BeginTransaction ();
 		foreach (int num in SelectedIds ()) {
@@ -1631,7 +1650,7 @@ public class MainWindow {
 
 	public void HandleEditSelectedTag (object sender, EventArgs ea)
 	{
-		Tag [] tags = this.tag_selection_widget.TagHighlight ();
+		Tag [] tags = this.tag_selection_widget.TagHighlight;
 		if (tags.Length != 1)
 			return;
 
@@ -1651,7 +1670,7 @@ public class MainWindow {
 
 	public void HandleMergeTagsCommand (object obj, EventArgs args)
 	{
-		Tag [] tags = this.tag_selection_widget.TagHighlight ();
+		Tag [] tags = this.tag_selection_widget.TagHighlight;
 		if (tags.Length < 2)
 			return;
 		
@@ -2113,7 +2132,7 @@ public class MainWindow {
 
 	public void HandleDeleteSelectedTagCommand (object sender, EventArgs args)
 	{
-		Tag [] tags = this.tag_selection_widget.TagHighlight ();
+		Tag [] tags = this.tag_selection_widget.TagHighlight;
 
 		System.Array.Sort (tags, new TagRemoveComparer ());
 
@@ -2315,17 +2334,21 @@ public class MainWindow {
 
 		case Preferences.GLASS_POSITION:
 			if (query.Count > 0) {
-				IBrowsableItem photo = group_selector.Adaptor.PhotoFromIndex ((int) val);
-				
-				if (photo != null)
-					JumpTo (query.IndexOf (photo));
+				// If the database has changed since this pref was saved, this could cause
+				// an exception to be thrown.
+				try {
+					IBrowsableItem photo = group_selector.Adaptor.PhotoFromIndex ((int) val);
+					
+					if (photo != null)
+						JumpTo (query.IndexOf (photo));
+				} catch (Exception e) {}
 			}
 			break;
 		case Preferences.SIDEBAR_POSITION:
 			if (main_hpaned.Position != (int) val)
 				main_hpaned.Position = (int) val;
 			break;
-		
+
 		case Preferences.ZOOM:
 			icon_view.Zoom = (double) val;
 			break;
@@ -2473,9 +2496,8 @@ public class MainWindow {
 
 		if (selected == null || selected.Length < 1)
 			return;
-		
+
 		string header = Catalog.GetPluralString ("Create New Version?", "Create New Versions?", selected.Length); 
-		
 		string msg = String.Format (Catalog.GetPluralString (
 				"Before launching {1}, should F-Spot create a new version of the selected photo to preserve the original?",
 				"Before launching {1}, should F-Spot create new versions of the selected photos to preserve the originals?", selected.Length),
@@ -2483,18 +2505,20 @@ public class MainWindow {
 
 		// FIXME add cancel button? add help button?
 		HigMessageDialog hmd = new HigMessageDialog(main_window, DialogFlags.DestroyWithParent, 
-								   MessageType.Question, Gtk.ButtonsType.None,
-								   header, msg);
+							    MessageType.Question, Gtk.ButtonsType.None,
+							    header, msg);
+
 		hmd.AddButton (Gtk.Stock.No, Gtk.ResponseType.No, false);
 		//hmd.AddButton (Gtk.Stock.Cancel, Gtk.ResponseType.Cancel, false);
 		hmd.AddButton (Gtk.Stock.Yes, Gtk.ResponseType.Yes, true);
 
 		Gtk.ResponseType response = Gtk.ResponseType.Cancel;
- 		try {
- 			response = (Gtk.ResponseType) hmd.Run();
- 		} finally {
- 			hmd.Destroy();
- 		}
+
+		try {
+			response = (Gtk.ResponseType) hmd.Run();
+		} finally {
+			hmd.Destroy ();
+		}
 
 		if (response == Gtk.ResponseType.Cancel)
 			return;
@@ -2504,15 +2528,18 @@ public class MainWindow {
 		GLib.List uri_list = new GLib.List (typeof (string));
 		foreach (Photo photo in selected) {
 			if (create_new_versions) {
-				// exception handling? Out of space, blah, blah
+				// FIXME exception handling? Out of space, blah, blah
 				uint version = photo.CreateNamedVersion (mime_application.Name, photo.DefaultVersionId, true);
 				photo.DefaultVersionId = version;
-				UpdateForVersionIdChange (version);
 			}
 
 			uri_list.Append (photo.DefaultVersionPath);
 		}
-		
+
+		if (create_new_versions) {
+			db.Photos.Commit (selected, new DbItemEventArgs (selected));
+		}
+
 		mime_application.Launch (uri_list);
 	}
 
@@ -2613,7 +2640,7 @@ public class MainWindow {
 
 	       // Add any new tags to the selected photos
 	       Category default_category = null;
-	       Tag [] selection = tag_selection_widget.TagHighlight ();
+	       Tag [] selection = tag_selection_widget.TagHighlight;
 	       if (selection.Length > 0) {
 		       if (selection [0] is Category)
 			       default_category = (Category) selection [0];
